@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2025 Colin Ian King.
+ * Copyright (C) 2022-2026 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -95,7 +95,7 @@ static int get_mount_info(stress_args_t *args)
 		mount_info[mounts].mount_path = shim_strdup(mount_path);
 		if (mount_info[mounts].mount_path == NULL) {
 			pr_dbg("%s: cannot allocate mountinfo mount path%s\n",
-				args->name, stress_get_memfree_str());
+				args->name, stress_memory_free_get());
 			free_mount_info(mounts);
 			mounts = -1;
 			break;
@@ -116,7 +116,7 @@ static int get_mount_info(stress_args_t *args)
 static int stress_handle_child(stress_args_t *args, void *context)
 {
 	const int mounts = *((int *)context);
-	const int bad_fd = stress_get_bad_fd();
+	const int bad_fd = stress_fs_bad_fd_get();
 	int rc = EXIT_SUCCESS;
 
 	do {
@@ -145,7 +145,7 @@ static int stress_handle_child(stress_args_t *args, void *context)
 			free(fhp);
 			break;
 		}
-		tmp = realloc(fhp, sizeof(*tmp) + fhp->handle_bytes);
+		tmp = (struct file_handle *)realloc(fhp, sizeof(*tmp) + fhp->handle_bytes);
 		if (UNLIKELY(!tmp)) {
 			free(fhp);
 			continue;
@@ -204,7 +204,7 @@ static int stress_handle_child(stress_args_t *args, void *context)
 
 		/* Exercise with large invalid size, EINVAL */
 		fhp->handle_bytes = 4096;
-		tmp = realloc(fhp, sizeof(*tmp) + fhp->handle_bytes);
+		tmp = (struct file_handle *)realloc(fhp, sizeof(*tmp) + fhp->handle_bytes);
 		if (UNLIKELY(!tmp)) {
 			(void)close(mount_fd);
 			free(fhp);
@@ -273,14 +273,14 @@ static int stress_handle(stress_args_t *args)
 		return EXIT_FAILURE;
 	}
 
-	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_proc_state_set(args->name, STRESS_STATE_SYNC_WAIT);
 	stress_sync_start_wait(args);
-	stress_set_proc_state(args->name, STRESS_STATE_RUN);
+	stress_proc_state_set(args->name, STRESS_STATE_RUN);
 
 	ret = stress_oomable_child(args, &mounts,
 		stress_handle_child, STRESS_OOMABLE_NORMAL);
 
-	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
+	stress_proc_state_set(args->name, STRESS_STATE_DEINIT);
 
 	free_mount_info(mounts);
 
