@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2022-2024 Colin Ian King.
+ * Copyright (C) 2022-2026 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -37,7 +37,8 @@ typedef struct {
 	bool trapped;
 } op_info_t;
 
-#if defined(STRESS_ARCH_ARM) &&	\
+#if defined(STRESS_ARCH_ARM) &&		\
+    defined(HAVE_SIGLONGJMP) &&		\
     defined(HAVE_ASM_ARM_TLBI)
 #define HAVE_PRIV_INSTR
 static void stress_arm_tlbi(void)
@@ -52,6 +53,7 @@ static op_info_t op_info[] =
 #endif
 
 #if defined(STRESS_ARCH_ALPHA) &&	\
+    defined(HAVE_SIGLONGJMP) &&		\
     (defined(HAVE_ASM_ALPHA_DRAINA) ||	\
      defined(HAVE_ASM_ALPHA_HALT))
 #define HAVE_PRIV_INSTR
@@ -85,6 +87,7 @@ static op_info_t op_info[] =
 #endif
 
 #if defined(STRESS_ARCH_HPPA) &&	\
+    defined(HAVE_SIGLONGJMP) &&		\
     (defined(HAVE_ASM_HPPA_DIAG) || 	\
      defined(HAVE_ASM_HPPA_RFI))
 #define HAVE_PRIV_INSTR
@@ -114,7 +117,39 @@ static op_info_t op_info[] =
 };
 #endif
 
+#if defined(STRESS_ARCH_LOONG64) &&	\
+    defined(HAVE_SIGLONGJMP)
+#define HAVE_PRIV_INSTR
+
+
+#if defined(HAVE_ASM_LOONG64_TLBRD)
+static void stress_loong64_tlbrd(void)
+{
+	__asm__ __volatile__("tlbrd");
+}
+#endif
+
+#if defined(HAVE_ASM_LOONG64_TLBSRCH)
+static void stress_loong64_tlbsrch(void)
+{
+	__asm__ __volatile__("tlbsrch");
+}
+#endif
+
+static op_info_t op_info[] =
+{
+#if defined(HAVE_ASM_LOONG64_TLBRD)
+	{ "tlbrd",	stress_loong64_tlbrd,	false, false },
+#endif
+#if defined(HAVE_ASM_LOONG64_TLBSRCH)
+	{ "tlbsrch",	stress_loong64_tlbsrch,	false, false },
+#endif
+};
+
+#endif
+
 #if defined(STRESS_ARCH_M68K) &&	\
+    defined(HAVE_SIGLONGJMP) &&		\
     defined(HAVE_ASM_M68K_EORI_SR)
 #define HAVE_PRIV_INSTR
 
@@ -130,6 +165,7 @@ static op_info_t op_info[] =
 #endif
 
 #if defined(STRESS_ARCH_MIPS) &&	\
+    defined(HAVE_SIGLONGJMP) &&		\
     defined(HAVE_ASM_MIPS_WAIT)
 #define HAVE_PRIV_INSTR
 static void stress_mips_wait(void)
@@ -143,7 +179,22 @@ static op_info_t op_info[] =
 };
 #endif
 
+#if defined(STRESS_ARCH_OR1K) &&	\
+    defined(HAVE_SIGLONGJMP)
+#define HAVE_PRIV_INSTR
+static void stress_or1k_lmtspr_0(void)
+{
+	__asm__ __volatile("l.mtspr r0,r31,0x0\n");
+}
+
+static op_info_t op_info[] =
+{
+	{ "l.mtspr r0,r1,0x0",	stress_or1k_lmtspr_0,	false, false },
+};
+#endif
+
 #if defined(STRESS_ARCH_PPC64) &&	\
+    defined(HAVE_SIGLONGJMP) &&		\
     defined(HAVE_ASM_PPC64_TLBIE)
 #define HAVE_PRIV_INSTR
 #define HAVE_PRIV_PAGE
@@ -152,9 +203,11 @@ static void *page;
 
 static void stress_ppc64_tlbie(void)
 {
-        unsigned long address = (unsigned long)page;
+	if (page != MAP_FAILED) {
+        	unsigned long int address = (unsigned long int)page;
 
-	__asm__ __volatile__("tlbie %0, 0" : : "r" (address) : "memory");
+		__asm__ __volatile__("tlbie %0, 0" : : "r" (address) : "memory");
+	}
 }
 
 static op_info_t op_info[] =
@@ -164,6 +217,7 @@ static op_info_t op_info[] =
 #endif
 
 #if defined(STRESS_ARCH_RISCV) &&	\
+    defined(HAVE_SIGLONGJMP) &&		\
     defined(HAVE_ASM_RISCV_SFENCE_VMA)
 #define HAVE_PRIV_INSTR
 static void stress_riscv_sfence_vma(void)
@@ -178,6 +232,7 @@ static op_info_t op_info[] =
 #endif
 
 #if defined(STRESS_ARCH_S390) &&	\
+    defined(HAVE_SIGLONGJMP) &&		\
     defined(HAVE_ASM_S390_PTLB)
 #define HAVE_PRIV_INSTR
 static void stress_s390_ptlb(void)
@@ -192,6 +247,7 @@ static op_info_t op_info[] =
 #endif
 
 #if defined(STRESS_ARCH_SH4) &&		\
+    defined(HAVE_SIGLONGJMP) &&		\
     (defined(HAVE_ASM_SH4_RTE) ||	\
      defined(HAVE_ASM_SH4_SLEEP))
 #define HAVE_PRIV_INSTR
@@ -222,11 +278,12 @@ static op_info_t op_info[] =
 #endif
 
 #if defined(STRESS_ARCH_SPARC) &&	\
+    defined(HAVE_SIGLONGJMP) &&		\
     defined(HAVE_ASM_SPARC_RDPR)
 #define HAVE_PRIV_INSTR
 static void stress_sparc_rdpr(void)
 {
-	unsigned long ver;
+	unsigned long int ver;
 
 	__asm__ __volatile__("rdpr %%ver, %0" : "=r" (ver));
 }
@@ -238,6 +295,7 @@ static op_info_t op_info[] =
 #endif
 
 #if defined(STRESS_ARCH_X86) &&		\
+    defined(HAVE_SIGLONGJMP) &&		\
     (defined(HAVE_ASM_X86_CLTS) ||	\
      defined(HAVE_ASM_X86_HLT) ||	\
      defined(HAVE_ASM_X86_INVD) || 	\
@@ -314,7 +372,7 @@ static void stress_x86_lmsw(void)
 #if defined(HAVE_ASM_X86_MOV_CR0)
 static void stress_x86_mov_cr0(void)
 {
-	unsigned long cr0;
+	unsigned long int cr0;
 
 	__asm__ __volatile__("mov %%cr0, %0" : "=r"(cr0) : : "memory");
 }
@@ -323,7 +381,7 @@ static void stress_x86_mov_cr0(void)
 #if defined(HAVE_ASM_X86_MOV_DR0)
 static void stress_x86_mov_dr0(void)
 {
-	unsigned long dr0;
+	unsigned long int dr0;
 
 	__asm__ __volatile__("mov %%dr0, %0" : "=r"(dr0) : : "memory");
 }
@@ -418,23 +476,25 @@ static sigjmp_buf jmp_env;
 static size_t idx = 0;
 static double t_start, duration, count;
 
-static inline void stress_sigsegv_handler(int signum)
+static void MLOCKED_TEXT NORETURN stress_sigsegv_handler(int signum)
 {
 	(void)signum;
 
 	duration += stress_time_now() - t_start;
 	count += 1.0;
-	op_info[idx].trapped = true;
+	if (idx < SIZEOF_ARRAY(op_info))
+		op_info[idx].trapped = true;
 	idx++;
 
-	siglongjmp(jmp_env, 1);
+	stress_signal_siglongjmp(signum, jmp_env, 1);
 }
 
 #if defined(SIGILL) ||	\
     defined(SIGBUS)
-static void stress_sigill_handler(int signum)
+static void MLOCKED_TEXT NORETURN stress_sigill_handler(int signum)
 {
-	op_info[idx].invalid = true;
+	if (idx < SIZEOF_ARRAY(op_info))
+		op_info[idx].invalid = true;
 	stress_sigsegv_handler(signum);
 }
 #endif
@@ -456,15 +516,17 @@ static int stress_priv_instr(stress_args_t *args)
 #if defined(HAVE_PRIV_PAGE)
 	page = mmap(NULL, args->page_size, PROT_READ | PROT_WRITE,
 			MAP_ANONYMOUS | MAP_SHARED, -1, 0);
+	if (page != MAP_FAILED)
+		stress_memory_anon_name_set(page, args->page_size, "priv-page");
 #endif
-	if (stress_sighandler(args->name, SIGSEGV, stress_sigsegv_handler, NULL))
+	if (stress_signal_handler(args->name, SIGSEGV, stress_sigsegv_handler, NULL))
 		return EXIT_NO_RESOURCE;
 #if defined(SIGILL)
-	if (stress_sighandler(args->name, SIGILL, stress_sigill_handler, NULL))
+	if (stress_signal_handler(args->name, SIGILL, stress_sigill_handler, NULL))
 		return EXIT_NO_RESOURCE;
 #endif
 #if defined(SIGBUS)
-	if (stress_sighandler(args->name, SIGBUS, stress_sigill_handler, NULL))
+	if (stress_signal_handler(args->name, SIGBUS, stress_sigill_handler, NULL))
 		return EXIT_NO_RESOURCE;
 #endif
 
@@ -473,28 +535,32 @@ static int stress_priv_instr(stress_args_t *args)
 		op_info[i].trapped = false;
 	}
 
-	stress_set_proc_state(args->name, STRESS_STATE_RUN);
+	stress_proc_state_set(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
+	stress_proc_state_set(args->name, STRESS_STATE_RUN);
 
 	do {
 		ret = sigsetjmp(jmp_env, 1);
-		if (!stress_continue(args))
+		if (UNLIKELY(!stress_continue(args)))
 			goto finish;
 	} while (ret == 1);
 
 	do {
-		if (idx >= SIZEOF_ARRAY(op_info))
-			idx = 0;
+		size_t tmp_idx = idx;
+
+		if (tmp_idx >= SIZEOF_ARRAY(op_info))
+			tmp_idx = 0;
 
 		stress_bogo_inc(args);
-		if (op_info[idx].op_func) {
+		if (op_info[tmp_idx].op_func) {
 			t_start = stress_time_now();
-			op_info[idx].op_func();
+			op_info[tmp_idx].op_func();
 		}
-		idx++;
+		idx = tmp_idx + 1;
 	} while (stress_continue(args));
 
 finish:
-        stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
+        stress_proc_state_set(args->name, STRESS_STATE_DEINIT);
 
 	/* Get an overestimated buffer length */
 	for (len = 0, i = 0; i < SIZEOF_ARRAY(op_info); i++) {
@@ -505,7 +571,7 @@ finish:
 	if (len > 0) {
 		char *str;
 
-		str = calloc(len, sizeof(*str));
+		str = (char *)calloc(len, sizeof(*str));
 		if (str) {
 			int unhandled = 0;
 
@@ -514,7 +580,7 @@ finish:
 				if (!op_info[i].trapped) {
 					unhandled++;
 					if (!*str)  {
-						(void)shim_strlcpy(str, op_info[i].instr, len);
+						(void)shim_strscpy(str, op_info[i].instr, len);
 					} else {
 						(void)shim_strlcat(str, ", ", len);
 						(void)shim_strlcat(str, op_info[i].instr, len);
@@ -528,7 +594,7 @@ finish:
 
 	rate = (count > 0.0) ? (duration / count) : 0.0;
 	stress_metrics_set(args, 0, "nanosecs per privileged op trap",
-		STRESS_DBL_NANOSECOND * rate, STRESS_HARMONIC_MEAN);
+		STRESS_DBL_NANOSECOND * rate, STRESS_METRIC_HARMONIC_MEAN);
 
 #if defined(HAVE_PRIV_PAGE)
 	if (page != MAP_FAILED)
@@ -542,18 +608,18 @@ finish:
 	return EXIT_SUCCESS;
 }
 
-stressor_info_t stress_priv_instr_info = {
+const stressor_info_t stress_priv_instr_info = {
 	.stressor = stress_priv_instr,
-	.class = CLASS_CPU,
+	.classifier = CLASS_CPU,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 
 #else
 
-stressor_info_t stress_priv_instr_info = {
+const stressor_info_t stress_priv_instr_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_CPU,
+	.classifier = CLASS_CPU,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "no privileged op-code test for this architecture"
