@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2014-2021 Canonical, Ltd.
- * Copyright (C) 2021-2025 Colin Ian King.
+ * Copyright (C) 2021-2026 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -103,7 +103,7 @@ static int stress_dirent_proc_prune(struct dirent **dlist, const int n)
 		if (dlist[i]) {
 			register bool ignore = false;
 
-			if (stress_is_dot_filename(dlist[i]->d_name))
+			if (stress_fs_filename_dotty(dlist[i]->d_name))
 				ignore = true;
 			else if (isdigit((unsigned char)dlist[i]->d_name[0])) {
 				/* only allow a small numeric files.. */
@@ -150,9 +150,9 @@ static int stress_proc_scandir(
 static uint32_t mixup_hash(const char *str)
 {
 	if (isdigit((int)str[0])) {
-		const uint32_t val = atol(str);
+		const uint32_t val = (uint32_t)atol(str);
 
-		return stress_reverse32(val ^ mixup);
+		return stress_bitops_reverse32(val ^ mixup);
 	}
 	return stress_hash_pjw(str) ^ mixup;
 
@@ -177,7 +177,7 @@ static void stress_proc_self_mem(stress_args_t *args, const int fd)
 {
 	uint8_t *page, *buf;
 	const uint8_t rnd = stress_mwc8();
-	const size_t page_size = stress_get_page_size();
+	const size_t page_size = stress_memory_page_size_get();
 	off_t offset;
 
 	buf = (uint8_t *)malloc(page_size);
@@ -190,7 +190,7 @@ static void stress_proc_self_mem(stress_args_t *args, const int fd)
 		free(buf);
 		return;
 	}
-	stress_set_vma_anon_name(page, page_size, "proc-self-mem");
+	stress_memory_anon_name_set(page, page_size, "proc-self-mem");
 	offset = (off_t)(uintptr_t)page;
 
 	(void)shim_memset(page, rnd, page_size);
@@ -202,7 +202,8 @@ static void stress_proc_self_mem(stress_args_t *args, const int fd)
 		if ((ret == (ssize_t)page_size) &&
 		    (buf[mem_offset] != page[mem_offset])) {
 			pr_inf("%s /proc/self/mem read/mmap failure at offset %p, mmap value 0x%2x vs read value 0x%2x\n",
-				args->name, page + mem_offset, page[mem_offset], buf[mem_offset]);
+				args->name, (void *)(page + mem_offset),
+				page[mem_offset], buf[mem_offset]);
 		}
 	}
 
@@ -240,19 +241,19 @@ static void stress_proc_mtrr(stress_args_t *args, const int fd)
 
 #if defined(HAVE_MTRR_SENTRY) &&	\
     defined(MTRRIOC_SET_ENTRY)
-		(void)memset(&sentry, 0, sizeof(sentry));
+		(void)shim_memset(&sentry, 0, sizeof(sentry));
 		sentry.base = gentry.base;
 		sentry.size = 0;
 		sentry.type = gentry.type;
 		VOID_RET(int, ioctl(fd, MTRRIOC_SET_ENTRY, &sentry));
 
-		(void)memset(&sentry, 0, sizeof(sentry));
+		(void)shim_memset(&sentry, 0, sizeof(sentry));
 		sentry.base = gentry.base;
 		sentry.size = gentry.size;
 		sentry.type = ~0;
 		VOID_RET(int, ioctl(fd, MTRRIOC_SET_ENTRY, &sentry));
 
-		(void)memset(&sentry, 0, sizeof(sentry));
+		(void)shim_memset(&sentry, 0, sizeof(sentry));
 		sentry.base = gentry.base;
 		sentry.size = ~4095;
 		sentry.type = gentry.type;
@@ -578,7 +579,7 @@ mmap_test:
 			ptr = (uint8_t *)stress_mmap_populate(NULL, page_size, PROT_READ,
 				MAP_SHARED | MAP_ANONYMOUS, fd, 0);
 			if (ptr != MAP_FAILED) {
-				stress_uint8_put(*ptr);
+				stress_put_uint8(*ptr);
 				(void)munmap((void *)ptr, page_size);
 			}
 
@@ -680,7 +681,7 @@ err:
 
 		/* /proc + ... */
 		if (len > 5) {
-			char *pptr = path + 5 + stress_mwc16modn(len - 5);
+			char *pptr = path + 5 + stress_mwcsizemodn(len - 5);
 
 			/* Skip over / */
 			while (*pptr == '/')
@@ -776,7 +777,7 @@ static void stress_proc_dir(
 	dlist = NULL;
 	n = stress_proc_scandir(path, &dlist, NULL, mixup_sort);
 	if (n <= 0) {
-		stress_dirent_list_free(dlist, n);
+		stress_fs_dirent_list_free(dlist, n);
 		return;
 	}
 
@@ -785,7 +786,7 @@ static void stress_proc_dir(
 		struct dirent *d = dlist[i];
 		unsigned char type;
 
-		if (stress_is_dot_filename(d->d_name)) {
+		if (stress_fs_filename_dotty(d->d_name)) {
 			free(d);
 			dlist[i] = NULL;
 			continue;
@@ -795,7 +796,7 @@ static void stress_proc_dir(
 		if ((type == SHIM_DT_REG) || (type == SHIM_DT_LNK)) {
 			ret = shim_pthread_spin_lock(&lock);
 			if (!ret) {
-				(void)stress_mk_filename(tmp, sizeof(tmp), path, d->d_name);
+				(void)stress_fs_make_filename(tmp, sizeof(tmp), path, d->d_name);
 				(void)shim_strscpy(proc_path, tmp, sizeof(proc_path));
 				(void)shim_pthread_spin_unlock(&lock);
 
@@ -808,7 +809,7 @@ static void stress_proc_dir(
 	}
 
 	if (!recurse) {
-		stress_dirent_list_free(dlist, n);
+		stress_fs_dirent_list_free(dlist, n);
 		return;
 	}
 
@@ -817,7 +818,7 @@ static void stress_proc_dir(
 		struct dirent *d = dlist[i];
 
 		if (d && (shim_dirent_type(path, d) == SHIM_DT_DIR)) {
-			(void)stress_mk_filename(tmp, sizeof(tmp), path, d->d_name);
+			(void)stress_fs_make_filename(tmp, sizeof(tmp), path, d->d_name);
 
 			free(d);
 			dlist[i] = NULL;
@@ -826,7 +827,7 @@ static void stress_proc_dir(
 			stress_bogo_inc(args);
 		}
 	}
-	stress_dirent_list_free(dlist, n);
+	stress_fs_dirent_list_free(dlist, n);
 }
 
 /*
@@ -845,7 +846,7 @@ static char *stress_random_pid(void)
 	mixup = stress_mwc32();
 	n = stress_proc_scandir("/proc", &dlist, NULL, mixup_sort);
 	if (n <= 0) {
-		stress_dirent_list_free(dlist, n);
+		stress_fs_dirent_list_free(dlist, n);
 		return path;
 	}
 
@@ -861,12 +862,12 @@ static char *stress_random_pid(void)
 		name = dlist[j]->d_name;
 
 		if (isdigit((unsigned char)name[0])) {
-			(void)stress_mk_filename(path, sizeof(path), "/proc", name);
+			(void)stress_fs_make_filename(path, sizeof(path), "/proc", name);
 			break;
 		}
 	}
 
-	stress_dirent_list_free(dlist, n);
+	stress_fs_dirent_list_free(dlist, n);
 	return path;
 }
 
@@ -902,7 +903,7 @@ static int stress_procfs(stress_args_t *args)
 	(void)shim_strscpy(proc_path, "/proc/self", sizeof(proc_path));
 
 	ctxt.args = args;
-	ctxt.writeable = !stress_check_capability(SHIM_CAP_IS_ROOT);
+	ctxt.writeable = !stress_capabilities_check(SHIM_CAP_IS_ROOT);
 
 	rc = shim_pthread_spin_init(&lock, PTHREAD_PROCESS_PRIVATE);
 	if (rc) {
@@ -918,9 +919,9 @@ static int stress_procfs(stress_args_t *args)
 				stress_proc_rw_thread, &ctxt);
 	}
 
-	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_proc_state_set(args->name, STRESS_STATE_SYNC_WAIT);
 	stress_sync_start_wait(args);
-	stress_set_proc_state(args->name, STRESS_STATE_RUN);
+	stress_proc_state_set(args->name, STRESS_STATE_RUN);
 
 	do {
 		size_t j = stress_mwc32() % n;
@@ -933,7 +934,7 @@ static int stress_procfs(stress_args_t *args)
 			if (UNLIKELY(!stress_continue(args)))
 				break;
 
-			stress_mk_filename(procfspath, sizeof(procfspath), "/proc", d->d_name);
+			stress_fs_make_filename(procfspath, sizeof(procfspath), "/proc", d->d_name);
 			type = shim_dirent_type("/proc", d);
 			if ((type == SHIM_DT_REG) || (type == SHIM_DT_LNK)) {
 				if (!shim_pthread_spin_lock(&lock)) {
@@ -967,7 +968,7 @@ static int stress_procfs(stress_args_t *args)
 		VOID_RET(int, shim_pthread_spin_unlock(&lock));
 	}
 
-	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
+	stress_proc_state_set(args->name, STRESS_STATE_DEINIT);
 
 	for (i = 0; i < MAX_PROCFS_THREADS; i++) {
 		if (ret[i] == 0) {
@@ -977,7 +978,7 @@ static int stress_procfs(stress_args_t *args)
 	}
 	(void)shim_pthread_spin_destroy(&lock);
 
-	stress_dirent_list_free(dlist, n);
+	stress_fs_dirent_list_free(dlist, n);
 
 	return EXIT_SUCCESS;
 }
