@@ -23,6 +23,7 @@
 #include "core-capabilities.h"
 #include "core-killpid.h"
 #include "core-net.h"
+#include "core-signal.h"
 
 #include <sys/ioctl.h>
 
@@ -75,7 +76,7 @@ static const stress_help_t help[] = {
  */
 static int stress_rawpkt_supported(const char *name)
 {
-	if (!stress_check_capability(SHIM_CAP_NET_RAW)) {
+	if (!stress_capabilities_check(SHIM_CAP_NET_RAW)) {
 		pr_inf_skip("%s stressor will be skipped, "
 			"need to be running with CAP_NET_RAW "
 			"rights for this stressor\n", name);
@@ -264,7 +265,7 @@ static void NORETURN OPTIMIZE3 stress_rawpkt_client(
 	int fd;
 
 	stress_parent_died_alarm();
-	(void)sched_settings_apply(true);
+	(void)stress_sched_settings_apply(true);
 
 	(void)shim_memset(&sadr, 0, sizeof(sadr));
 	(void)shim_memset(buf, 0, sizeof(buf));
@@ -300,7 +301,7 @@ static void NORETURN OPTIMIZE3 stress_rawpkt_client(
 		ssize_t n;
 
 		ip->id = htons(id++);
-		ip->check = stress_ipv4_checksum((uint16_t *)ip, sizeof(struct iphdr) + sizeof(struct udphdr));
+		ip->check = stress_net_ipv4_checksum((uint16_t *)ip, sizeof(struct iphdr) + sizeof(struct udphdr));
 
 		n = sendto(fd, buf, sizeof(struct ethhdr) + ip->tot_len, 0, (struct sockaddr *)&sadr, sizeof(sadr));
 		if (UNLIKELY(n < 0)) {
@@ -348,7 +349,7 @@ static int OPTIMIZE3 stress_rawpkt_server(
 	const ssize_t min_size = sizeof(struct ethhdr) + sizeof(struct iphdr) + sizeof(struct udphdr);
 	double t_start, duration, bytes = 0.0, rate;
 
-	if (stress_sig_stop_stressing(args->name, SIGALRM) < 0) {
+	if (stress_signal_stop_stressing(args->name, SIGALRM) < 0) {
 		rc = EXIT_FAILURE;
 		goto die;
 	}
@@ -432,13 +433,6 @@ die:
 	return rc;
 }
 
-static void stress_sock_sigpipe_handler(int signum)
-{
-	(void)signum;
-
-	stress_continue_set_flag(false);
-}
-
 /*
  *  stress_rawpkt
  *	stress raw socket I/O UDP packet send/receive
@@ -451,11 +445,16 @@ static int stress_rawpkt(stress_args_t *args)
 	struct ifreq hwaddr, ifaddr, idx;
 	int rawpkt_rxring = 0;
 
-	if (stress_sigchld_set_handler(args) < 0)
+	if (stress_signal_sigchld_handler(args) < 0)
 		return EXIT_NO_RESOURCE;
 
-	(void)stress_get_setting("rawpkt-port", &rawpkt_port);
-	(void)stress_get_setting("rawpkt-rxring", &rawpkt_rxring);
+	(void)stress_setting_get("rawpkt-port", &rawpkt_port);
+	if (!stress_setting_get("rawpkt-rxring", &rawpkt_rxring)) {
+		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
+			rawpkt_rxring = 16;
+		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
+			rawpkt_rxring = 1;
+	}
 
 	if ((rawpkt_rxring & (rawpkt_rxring - 1)) != 0) {
 		(void)pr_inf("%s: --rawpkt-rxring value %d is not "
@@ -463,9 +462,8 @@ static int stress_rawpkt(stress_args_t *args)
 		rawpkt_rxring = 0;
 	}
 	rawpkt_port += args->instance;
-	if (rawpkt_port > MAX_PORT)
-		rawpkt_port -= (MAX_PORT - MIN_PORT + 1); /* Wrap round */
-	reserved_port = stress_net_reserve_ports(rawpkt_port, rawpkt_port);
+	rawpkt_port = stress_net_port_wraparound(rawpkt_port);
+	reserved_port = stress_net_reserve_ports(args, rawpkt_port, rawpkt_port);
 	if (reserved_port < 0) {
 		pr_inf_skip("%s: cannot reserve port %d, skipping stressor\n",
 			args->name, rawpkt_port);
@@ -476,7 +474,7 @@ static int stress_rawpkt(stress_args_t *args)
 	pr_dbg("%s: process [%d] using socket port %d\n",
 		args->name, (int)args->pid, rawpkt_port);
 
-	if (stress_sighandler(args->name, SIGPIPE, stress_sock_sigpipe_handler, NULL) < 0)
+	if (stress_signal_handler(args->name, SIGPIPE, stress_signal_stop_flag_handler, NULL) < 0)
 		return EXIT_NO_RESOURCE;
 
 	if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
@@ -512,11 +510,11 @@ static int stress_rawpkt(stress_args_t *args)
 	}
 	(void)close(fd);
 
-	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_proc_state_set(args->name, STRESS_STATE_SYNC_WAIT);
 	stress_sync_start_wait(args);
-	stress_set_proc_state(args->name, STRESS_STATE_RUN);
+	stress_proc_state_set(args->name, STRESS_STATE_RUN);
 again:
-	parent_cpu = stress_get_cpu();
+	parent_cpu = stress_cpu_get();
 	pid = fork();
 	if (pid < 0) {
 		if (stress_redo_fork(args, errno))
@@ -529,16 +527,16 @@ again:
 			args->name, errno, strerror(errno));
 		return rc;
 	} else if (pid == 0) {
-		stress_set_proc_state(args->name, STRESS_STATE_RUN);
-		stress_set_make_it_fail();
-		(void)stress_change_cpu(args, parent_cpu);
+		stress_proc_state_set(args->name, STRESS_STATE_RUN);
+		stress_make_it_fail_set();
+		(void)stress_affinity_change_cpu(args, parent_cpu);
 		stress_rawpkt_client(args, &hwaddr, &ifaddr, &idx, rawpkt_port);
 	} else {
 		rc = stress_rawpkt_server(args, &ifaddr, rawpkt_port, rawpkt_rxring);
 		(void)stress_kill_pid_wait(pid, NULL);
 	}
 finish:
-	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
+	stress_proc_state_set(args->name, STRESS_STATE_DEINIT);
 
 	return rc;
 }
