@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2025 Colin Ian King.
+ * Copyright (C) 2022-2026 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,6 +22,7 @@
 #include "core-builtin.h"
 #include "core-killpid.h"
 #include "core-net.h"
+#include "core-signal.h"
 
 #if defined(HAVE_SYS_UN_H)
 #include <sys/un.h>
@@ -43,7 +44,9 @@ UNEXPECTED
 
 #define DEFAULT_SCTP_PORT	(9000)
 
-#define SOCKET_BUF		(8192)	/* Socket I/O buffer size */
+#define MIN_SCTP_MAX_SIZE	(16)
+#define MAX_SCTP_MAX_SIZE	(65536)
+#define DEFAULT_SCTP_MAX_SIZE	(8192)
 
 typedef struct {
 	const int	sched_type;
@@ -51,13 +54,14 @@ typedef struct {
 } stress_sctp_sched_t;
 
 static const stress_help_t help[] = {
-	{ NULL,	"sctp N",	 "start N workers performing SCTP send/receives " },
-	{ NULL,	"sctp-domain D", "specify sctp domain, default is ipv4" },
-	{ NULL,	"sctp-if I",	 "use network interface I, e.g. lo, eth0, etc." },
-	{ NULL,	"sctp-ops N",	 "stop after N SCTP bogo operations" },
-	{ NULL,	"sctp-port P",	 "use SCTP ports P to P + number of workers - 1" },
-	{ NULL, "sctp-sched S",	 "specify sctp scheduler" },
-	{ NULL,	NULL, 		 NULL }
+	{ NULL,	"sctp N",	   "start N workers performing SCTP send/receives " },
+	{ NULL,	"sctp-domain D",   "specify sctp domain, default is ipv4" },
+	{ NULL,	"sctp-if I",	   "use network interface I, e.g. lo, eth0, etc." },
+	{ NULL, "sctp-max-size N", "specify maximum size of sctp data" },
+	{ NULL,	"sctp-ops N",	   "stop after N SCTP bogo operations" },
+	{ NULL,	"sctp-port P",	   "use SCTP ports P to P + number of workers - 1" },
+	{ NULL, "sctp-sched S",	   "specify sctp scheduler" },
+	{ NULL,	NULL,              NULL }
 };
 
 #if defined(HAVE_LIB_SCTP) &&	\
@@ -97,10 +101,11 @@ static const char *stress_sctp_sched(const size_t i)
 static int sctp_domain_mask = DOMAIN_INET | DOMAIN_INET6;
 
 static const stress_opt_t opts[] = {
-	{ OPT_sctp_domain, "sctp-domain", TYPE_ID_INT_DOMAIN, 0, 0, &sctp_domain_mask },
-	{ OPT_sctp_if,     "sctp-if",     TYPE_ID_STR, 0, 0, NULL },
-	{ OPT_sctp_port,   "sctp-port",   TYPE_ID_INT_PORT, MIN_PORT, MAX_PORT, NULL },
-	{ OPT_sctp_sched,  "sctp-sched",  TYPE_ID_SIZE_T_METHOD, 0, 0, stress_sctp_sched },
+	{ OPT_sctp_domain,   "sctp-domain",   TYPE_ID_INT_DOMAIN, 0, 0, &sctp_domain_mask },
+	{ OPT_sctp_if,       "sctp-if",       TYPE_ID_STR, 0, 0, NULL },
+	{ OPT_sctp_max_size, "sctp-max-size", TYPE_ID_SIZE_T, MIN_SCTP_MAX_SIZE, MAX_SCTP_MAX_SIZE, NULL },
+	{ OPT_sctp_port,     "sctp-port",   TYPE_ID_INT_PORT, MIN_PORT, MAX_PORT, NULL },
+	{ OPT_sctp_sched,    "sctp-sched",  TYPE_ID_SIZE_T_METHOD, 0, 0, (void *)stress_sctp_sched },
 	END_OPT,
 };
 
@@ -171,6 +176,10 @@ static void stress_sctp_sockopts(const int fd)
 #endif
 #if defined(SCTP_MAXSEG)
 	{
+		/*
+		 *  try using the wrong type, kernel will
+		 *  complain, do this once
+		 */
 		static bool once = false;
 
 		if (once == false) {
@@ -206,6 +215,10 @@ static void stress_sctp_sockopts(const int fd)
 #endif
 #if defined(SCTP_MAX_BURST)
 	{
+		/*
+		 *  try using the wrong type, kernel will
+		 *  complain, do this once
+		 */
 		static bool once = false;
 
 		if (once == false) {
@@ -294,8 +307,18 @@ static void stress_sctp_sockopts(const int fd)
 	UNEXPECTED
 #endif
 #if defined(SCTP_EVENT) &&	\
-    defined(HAVE_SCTP_EVENT_SUBSCRIBE)
-	STRESS_SCTP_SOCKOPT(SCTP_EVENT, struct sctp_event_subscribe)
+    defined(HAVE_SCTP_EVENT)
+	{
+		typedef int32_t	shim_sctp_assoc_t;
+
+		struct shim_sctp_event {
+			shim_sctp_assoc_t se_assoc_id;
+			uint16_t	se_type;
+			uint8_t		se_on;
+		};
+
+		STRESS_SCTP_SOCKOPT(SCTP_EVENT, struct shim_sctp_event)
+	}
 #endif
 #if defined(SCTP_ASCONF_SUPPORTED) &&	\
     defined(HAVE_SCTP_ASSOC_VALUE)
@@ -351,10 +374,10 @@ static int OPTIMIZE3 stress_sctp_client(
 	(void)sctp_sched_type;
 
 	stress_parent_died_alarm();
-	(void)sched_settings_apply(true);
+	(void)stress_sched_settings_apply(true);
 
 	do {
-		char ALIGN64 buf[SOCKET_BUF];
+		char ALIGN64 buf[MAX_SCTP_MAX_SIZE];
 		int fd;
 		int retries = 0;
 		socklen_t addr_len = 0;
@@ -377,9 +400,9 @@ retry:
 			return EXIT_FAILURE;
 		}
 
-		if (UNLIKELY(stress_set_sockaddr_if(args->name, args->instance, mypid,
-				sctp_domain, sctp_port, sctp_if,
-				&addr, &addr_len, NET_ADDR_LOOPBACK) < 0)) {
+		if (UNLIKELY(stress_net_sockaddr_if_set(args->name, args->instance, mypid,
+							sctp_domain, sctp_port, sctp_if,
+							&addr, &addr_len, NET_ADDR_LOOPBACK) < 0)) {
 			(void)close(fd);
 			return EXIT_FAILURE;
 		}
@@ -419,7 +442,7 @@ retry:
 #endif
 
 		do {
-			int flags;
+			int flags = 0;
 			struct sctp_sndrcvinfo sndrcvinfo;
 			ssize_t n;
 
@@ -428,15 +451,15 @@ retry:
 			if (UNLIKELY(n <= 0))
 				break;
 			if (n >= (ssize_t)sizeof(pid_t)) {
-				const pid_t *pidptr = (pid_t *)buf;
-				const pid_t pid = *pidptr;
+				pid_t pid;
 
+				shim_memcpy(&pid, buf, sizeof(pid));
 				if (UNLIKELY(pid != mypid)) {
-					pr_fail("%s: server received unexpected data "
+					pr_fail("%s: client received unexpected data "
 						"contents, got 0x%" PRIxMAX ", "
 						"expected 0x%" PRIxMAX "\n",
-						args->name, (intmax_t)pid,
-						(intmax_t)mypid);
+						args->name, (uintmax_t)pid,
+						(uintmax_t)mypid);
 					rc = EXIT_FAILURE;
 					break;
 				}
@@ -447,6 +470,7 @@ retry:
 	} while (stress_continue(args));
 
 #if defined(AF_UNIX) &&		\
+    defined(HAVE_SYS_UN_H) &&	\
     defined(HAVE_SOCKADDR_UN)
 	if (sctp_domain == AF_UNIX) {
 		const struct sockaddr_un *addr_un = (struct sockaddr_un *)addr;
@@ -469,19 +493,21 @@ static int OPTIMIZE3 stress_sctp_server(
 	const int sctp_port,
 	const int sctp_domain,
 	const int sctp_sched_type,
-	const char *sctp_if)
+	const char *sctp_if,
+	const size_t sctp_max_size)
 {
-	char ALIGN64 buf[SOCKET_BUF];
+	char ALIGN64 buf[MAX_SCTP_MAX_SIZE];
 	int fd;
 	int so_reuseaddr = 1;
 	socklen_t addr_len = 0;
 	struct sockaddr *addr = NULL;
 	int rc = EXIT_SUCCESS;
 	int idx = 0;
+	const size_t sctp_min_size = (sctp_max_size & 0xf) + 16;
 
 	(void)sctp_sched_type;
 
-	if (stress_sig_stop_stressing(args->name, SIGALRM)) {
+	if (stress_signal_stop_stressing(args->name, SIGALRM)) {
 		rc = EXIT_FAILURE;
 		goto die;
 	}
@@ -506,8 +532,9 @@ static int OPTIMIZE3 stress_sctp_server(
 		rc = EXIT_FAILURE;
 		goto die_close;
 	}
-	if (stress_set_sockaddr_if(args->name, args->instance, mypid,
-		sctp_domain, sctp_port, sctp_if, &addr, &addr_len, NET_ADDR_ANY) < 0) {
+	if (stress_net_sockaddr_if_set(args->name, args->instance, mypid,
+				       sctp_domain, sctp_port, sctp_if,
+				       &addr, &addr_len, NET_ADDR_ANY) < 0) {
 		rc = EXIT_FAILURE;
 		goto die_close;
 	}
@@ -523,13 +550,13 @@ static int OPTIMIZE3 stress_sctp_server(
 		rc = EXIT_FAILURE;
 		goto die_close;
 	}
-#if defined(TCP_NODELAY)
+#if defined(SCTP_NODELAY)
 	{
 		int one = 1;
 
 		if (g_opt_flags & OPT_FLAGS_SOCKET_NODELAY) {
-			if (setsockopt(fd, SOL_TCP, TCP_NODELAY, &one, sizeof(one)) < 0) {
-				pr_inf("%s: setsockopt TCP_NODELAY "
+			if (setsockopt(fd, SOL_TCP, SCTP_NODELAY, &one, sizeof(one)) < 0) {
+				pr_inf("%s: setsockopt SCTP_NODELAY "
 					"failed and disabled, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 				g_opt_flags &= ~OPT_FLAGS_SOCKET_NODELAY;
@@ -560,17 +587,15 @@ static int OPTIMIZE3 stress_sctp_server(
 		if (LIKELY(sfd >= 0)) {
 			size_t i;
 			const int c = stress_ascii32[idx++ & 0x1f];
-			pid_t *pidptr = (pid_t *)buf;
 
-			(void)shim_memset(buf, c, sizeof(buf));
-			*pidptr = mypid;
-			for (i = 16; i < sizeof(buf); i += 16) {
+			(void)shim_memset(buf + sizeof(mypid), c, sizeof(buf) - sizeof(mypid));
+			(void)shim_memcpy(buf, &mypid, sizeof(mypid));
+			for (i = sctp_min_size; i <= sctp_max_size; i += 16) {
 				ssize_t ret = sctp_sendmsg(sfd, buf, i,
 						NULL, 0, 0, 0,
 						LOCALTIME_STREAM, 0, 0);
 				if (UNLIKELY(ret < 0))
 					break;
-
 				stress_bogo_inc(args);
 			}
 			stress_sctp_sockopts(sfd);
@@ -582,6 +607,7 @@ die_close:
 	(void)close(fd);
 die:
 #if defined(AF_UNIX) &&		\
+    defined(HAVE_SYS_UN_H) &&	\
     defined(HAVE_SOCKADDR_UN)
 	if (addr && (sctp_domain == AF_UNIX)) {
 		const struct sockaddr_un *addr_un = (struct sockaddr_un *)addr;
@@ -612,16 +638,23 @@ static int stress_sctp(stress_args_t *args)
 	int sctp_domain = AF_INET;
 	int sctp_sched_type = -1; /* undefined */
 	size_t sctp_sched = 1; /* default to fcfs */
+	size_t sctp_max_size = DEFAULT_SCTP_MAX_SIZE;
 	int ret, reserved_port, parent_cpu;
 	char *sctp_if = NULL;
 
-	if (stress_sigchld_set_handler(args) < 0)
+	if (stress_signal_sigchld_handler(args) < 0)
 		return EXIT_NO_RESOURCE;
 
-	(void)stress_get_setting("sctp-domain", &sctp_domain);
-	(void)stress_get_setting("sctp-if", &sctp_if);
-	(void)stress_get_setting("sctp-port", &sctp_port);
-	if (stress_get_setting("sctp-sched", &sctp_sched)) {
+	(void)stress_setting_get("sctp-domain", &sctp_domain);
+	(void)stress_setting_get("sctp-if", &sctp_if);
+	if (!stress_setting_get("sctp-max-size", &sctp_max_size)) {
+		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
+			sctp_max_size = MAX_SCTP_MAX_SIZE;
+		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
+			sctp_max_size = MIN_SCTP_MAX_SIZE;
+	}
+	(void)stress_setting_get("sctp-port", &sctp_port);
+	if (stress_setting_get("sctp-sched", &sctp_sched)) {
 #if defined(HAVE_SCTP_SCHED_TYPE) &&	\
     defined(HAVE_SCTP_ASSOC_VALUE)
 		sctp_sched_type = stress_sctp_scheds[sctp_sched].sched_type;
@@ -641,13 +674,12 @@ static int stress_sctp(stress_args_t *args)
 		}
 	}
 
-	if (stress_sighandler(args->name, SIGPIPE, stress_sctp_sigpipe, NULL) < 0)
+	if (stress_signal_handler(args->name, SIGPIPE, stress_sctp_sigpipe, NULL) < 0)
 		return EXIT_FAILURE;
 
 	sctp_port += args->instance;
-	if (sctp_port > MAX_PORT)
-		sctp_port -= (MAX_PORT - MIN_PORT + 1);
-	reserved_port = stress_net_reserve_ports(sctp_port, sctp_port);
+	sctp_port = stress_net_port_wraparound(sctp_port);
+	reserved_port = stress_net_reserve_ports(args, sctp_port, sctp_port);
 	if (reserved_port < 0) {
 		pr_inf_skip("%s: cannot reserve port %d, skipping stressor\n",
 			args->name, sctp_port);
@@ -660,11 +692,11 @@ static int stress_sctp(stress_args_t *args)
 
 	ret = EXIT_FAILURE;
 
-	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_proc_state_set(args->name, STRESS_STATE_SYNC_WAIT);
 	stress_sync_start_wait(args);
-	stress_set_proc_state(args->name, STRESS_STATE_RUN);
+	stress_proc_state_set(args->name, STRESS_STATE_RUN);
 again:
-	parent_cpu = stress_get_cpu();
+	parent_cpu = stress_cpu_get();
 	pid = fork();
 	if (pid < 0) {
 		if (stress_redo_fork(args, errno))
@@ -675,13 +707,17 @@ again:
 			args->name, errno, strerror(errno));
 		return EXIT_FAILURE;
 	} else if (pid == 0) {
-		 (void)stress_change_cpu(args, parent_cpu);
-		ret = stress_sctp_client(args, mypid, sctp_port, sctp_domain, sctp_sched_type, sctp_if);
+		stress_proc_state_set(args->name, STRESS_STATE_RUN);
+		stress_make_it_fail_set();
+		 (void)stress_affinity_change_cpu(args, parent_cpu);
+		ret = stress_sctp_client(args, mypid, sctp_port, sctp_domain,
+					 sctp_sched_type, sctp_if);
 		_exit(ret);
 	} else {
 		int status;
 
-		ret = stress_sctp_server(args, mypid, sctp_port, sctp_domain, sctp_sched_type, sctp_if);
+		ret = stress_sctp_server(args, mypid, sctp_port, sctp_domain,
+					 sctp_sched_type, sctp_if, sctp_max_size);
 		(void)stress_kill_pid_wait(pid, &status);
 		if (WIFEXITED(status)) {
 			if (WEXITSTATUS(status) != EXIT_SUCCESS) {
@@ -694,7 +730,7 @@ finish:
 	if (sigpipe_count)
 		pr_dbg("%s: caught %" PRIu64 " SIGPIPE signals\n", args->name, sigpipe_count);
 
-	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
+	stress_proc_state_set(args->name, STRESS_STATE_DEINIT);
 	stress_net_release_ports(sctp_port, sctp_port);
 
 	return ret;
