@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2026 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,7 +18,16 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
 #include "core-net.h"
+#include "core-out-of-memory.h"
+#include "core-signal.h"
+
+#include <sys/ioctl.h>
+
+#if defined(HAVE_SYS_SELECT_H)
+#include <sys/select.h>
+#endif
 
 #if defined(HAVE_SYS_UN_H)
 #include <sys/un.h>
@@ -26,33 +35,20 @@
 UNEXPECTED
 #endif
 
-#define MIN_SOCKET_FD_PORT	(1024)
-#define MAX_SOCKET_FD_PORT	(65535)
 #define DEFAULT_SOCKET_FD_PORT	(8000)
 
 static const stress_help_t help[] = {
 	{ NULL,	"sockfd N",	 "start N workers sending file descriptors over sockets" },
 	{ NULL,	"sockfd-ops N",	 "stop after N sockfd bogo operations" },
 	{ NULL,	"sockfd-port P", "use socket fd ports P to P + number of workers - 1" },
+	{ NULL,	"sockfd-reuse",	 "reuse file descriptors between sender and receiver" },
 	{ NULL,	NULL,		 NULL }
 };
 
-/*
- *  stress_set_socket_fd_port()
- *	set port to use
- */
-static int stress_set_socket_fd_port(const char *opt)
-{
-	int socket_fd_port;
-
-	stress_set_net_port("sockfd-port", opt,
-		MIN_SOCKET_FD_PORT, MAX_SOCKET_FD_PORT, &socket_fd_port);
-	return stress_set_setting("sockfd-port", TYPE_ID_INT, &socket_fd_port);
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_sockfd_port,	stress_set_socket_fd_port },
-	{ 0,			NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_sockfd_port,  "sockfd-port",  TYPE_ID_INT_PORT, MIN_PORT, MAX_PORT, NULL },
+	{ OPT_sockfd_reuse, "sockfd-reuse", TYPE_ID_BOOL, 0, 1, NULL },
+	END_OPT,
 };
 
 #if defined(__linux__)
@@ -63,10 +59,10 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
  *  stress_socket_fd_send()
  *	send a fd (fd_send) over a socket fd
  */
-static inline ssize_t stress_socket_fd_sendmsg(const int fd, const int fd_send)
+static inline ssize_t stress_socket_fd_send(const int fd, const int fd_send)
 {
 	struct iovec iov;
-	struct msghdr msg;
+	struct msghdr msg ALIGN64;
 	struct cmsghdr *cmsg;
 	int *ptr;
 
@@ -76,11 +72,11 @@ static inline ssize_t stress_socket_fd_sendmsg(const int fd, const int fd_send)
 	iov.iov_base = msg_data;
 	iov.iov_len = 1;
 
-	(void)memset(&msg, 0, sizeof(msg));
+	(void)shim_memset(&msg, 0, sizeof(msg));
 	msg.msg_iov = &iov;
 	msg.msg_iovlen = 1;
 
-	(void)memset(ctrl, 0, sizeof(ctrl));
+	(void)shim_memset(ctrl, 0, sizeof(ctrl));
 	msg.msg_control = ctrl;
 	msg.msg_controllen = sizeof(ctrl);
 
@@ -101,7 +97,7 @@ static inline ssize_t stress_socket_fd_sendmsg(const int fd, const int fd_send)
 static inline int stress_socket_fd_recv(const int fd)
 {
 	struct iovec iov;
-	struct msghdr msg;
+	struct msghdr msg ALIGN64;
 	struct cmsghdr *cmsg;
 	char msg_data[1] = { 0 };
 	char ctrl[CMSG_SPACE(sizeof(int))];
@@ -109,19 +105,19 @@ static inline int stress_socket_fd_recv(const int fd)
 	iov.iov_base = msg_data;
 	iov.iov_len = 1;
 
-	(void)memset(&msg, 0, sizeof(msg));
+	(void)shim_memset(&msg, 0, sizeof(msg));
 	msg.msg_iov = &iov;
 	msg.msg_iovlen = 1;
 
-	(void)memset(ctrl, 0, sizeof(ctrl));
+	(void)shim_memset(ctrl, 0, sizeof(ctrl));
 	msg.msg_control = ctrl;
 	msg.msg_controllen = sizeof(ctrl);
 
-	if (recvmsg(fd, &msg, 0) <= 0)
-		return -errno;
-	if (msg_data[0] != MSG_ID)
+	if (UNLIKELY(recvmsg(fd, &msg, 0) <= 0))
 		return -1;
-	if ((msg.msg_flags & MSG_CTRUNC) == MSG_CTRUNC)
+	if (UNLIKELY(msg_data[0] != MSG_ID))
+		return -1;
+	if (UNLIKELY((msg.msg_flags & MSG_CTRUNC) == MSG_CTRUNC))
 		return -1;
 
 	cmsg = CMSG_FIRSTHDR(&msg);
@@ -140,92 +136,103 @@ static inline int stress_socket_fd_recv(const int fd)
  *  stress_socket_client()
  *	client reader
  */
-static int stress_socket_client(
-	const stress_args_t *args,
+static int OPTIMIZE3 stress_socket_client(
+	stress_args_t *args,
 	const pid_t mypid,
 	const ssize_t max_fd,
 	const int socket_fd_port,
+	const bool socket_fd_reuse,
 	int *fds,
 	const size_t fds_size)
 {
 	struct sockaddr *addr = NULL;
 
 	stress_parent_died_alarm();
-	(void)sched_settings_apply(true);
+	(void)stress_sched_settings_apply(true);
 
 	do {
 		int fd, retries = 0;
-		ssize_t i, n;
+		ssize_t n;
 		socklen_t addr_len = 0;
 		int so_reuseaddr = 1;
 
-		(void)memset(fds, 0, fds_size);
+		(void)shim_memset(fds, 0, fds_size);
 retry:
-		if (!keep_stressing_flag())
-			return EXIT_FAILURE;
+		if (UNLIKELY(!stress_continue_flag()))
+			return EXIT_NO_RESOURCE;
 
-		if ((fd = socket(AF_UNIX, SOCK_STREAM, 0)) < 0) {
+		if (UNLIKELY((fd = socket(AF_UNIX, SOCK_STREAM, 0)) < 0)) {
 			pr_fail("%s: socket failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			return EXIT_FAILURE;
 		}
-		if (setsockopt(fd, SOL_SOCKET, SO_REUSEADDR,
-				&so_reuseaddr, sizeof(so_reuseaddr)) < 0) {
+		if (UNLIKELY(setsockopt(fd, SOL_SOCKET, SO_REUSEADDR,
+				&so_reuseaddr, sizeof(so_reuseaddr)) < 0)) {
 			(void)close(fd);
 			pr_fail("%s: setsockopt SO_REUSEADDR failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			return EXIT_FAILURE;
 		}
 
-		if (stress_set_sockaddr(args->name, args->instance, mypid,
-				AF_UNIX, socket_fd_port,
-				&addr, &addr_len, NET_ADDR_ANY) < 0) {
+		if (UNLIKELY(stress_net_sockaddr_set(args->name, args->instance,
+						     mypid, AF_UNIX, socket_fd_port,
+						     &addr, &addr_len, NET_ADDR_ANY) < 0)) {
+			(void)close(fd);
 			return EXIT_FAILURE;
 		}
-		if (connect(fd, addr, addr_len) < 0) {
+		if (UNLIKELY(connect(fd, addr, addr_len) < 0)) {
 			(void)close(fd);
-			(void)shim_usleep(10000);
-			retries++;
-			if (retries > 100) {
+			if (retries++ > 100) {
 				/* Give up.. */
 				pr_fail("%s: connect failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
-				return EXIT_FAILURE;
+				return EXIT_NO_RESOURCE;
 			}
+			(void)shim_usleep(10000);
 			goto retry;
 		}
 
-		if (!keep_stressing_flag())
+		if (UNLIKELY(!stress_continue_flag())) {
+			(void)close(fd);
 			return EXIT_SUCCESS;
+		}
 
-		for (n = 0; keep_stressing(args) && (n < max_fd); n++) {
-			int rc, nbytes;
-
+		for (n = 0; LIKELY(stress_continue(args) && (n < max_fd)); n++) {
 			fds[n] = stress_socket_fd_recv(fd);
 			if (fds[n] < 0)
 				continue;
 
-			/* Attempt to read a byte from the fd */
-			rc = ioctl(fds[n], FIONREAD, &nbytes);
-			if ((rc == 0) && (nbytes >= 1)) {
-				char data;
+#if defined(HAVE_SELECT)
+			if (socket_fd_reuse)
+				(void)stress_socket_fd_send(fd, fds[n]);
+#else
+			(void)socket_fd_reuse;
+#endif
 
-				VOID_RET(ssize_t, read(fds[n], &data, sizeof(data)));
+#if defined(FIONREAD)
+			{
+				int rc, nbytes;
+
+				/* Attempt to read a byte from the fd */
+				rc = ioctl(fds[n], FIONREAD, &nbytes);
+				if ((rc == 0) && (nbytes >= 1)) {
+					char data;
+
+					VOID_RET(ssize_t, read(fds[n], &data, sizeof(data)));
+				}
 			}
+#endif
 		}
 
-		for (i = 0; i < n; i++) {
-			if (fds[i] >= 0)
-				(void)close(fds[i]);
-		}
-
+		stress_fs_close_fds(fds, n);
 		(void)shutdown(fd, SHUT_RDWR);
 		(void)close(fd);
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
-#if defined(HAVE_SOCKADDR_UN)
+#if defined(HAVE_SYS_UN_H) &&	\
+    defined(HAVE_SOCKADDR_UN)
 	if (addr) {
-		struct sockaddr_un *addr_un = (struct sockaddr_un *)addr;
+		const struct sockaddr_un *addr_un = (struct sockaddr_un *)addr;
 
 		(void)shim_unlink(addr_un->sun_path);
 	}
@@ -240,22 +247,22 @@ retry:
  *  stress_socket_server()
  *	server writer
  */
-static int stress_socket_server(
-	const stress_args_t *args,
-	const pid_t pid,
+static int OPTIMIZE3 stress_socket_server(
+	stress_args_t *args,
 	const pid_t ppid,
 	const ssize_t max_fd,
-	const int socket_fd_port)
+	const int socket_fd_port,
+	const bool socket_fd_reuse)
 {
-	int fd, status;
+	int fd;
 	int so_reuseaddr = 1;
 	socklen_t addr_len = 0;
 	struct sockaddr *addr = NULL;
 	uint64_t msgs = 0;
 	int rc = EXIT_SUCCESS;
-	const int bad_fd = stress_get_bad_fd();
+	const int bad_fd = stress_fs_bad_fd_get();
 
-	if (stress_sig_stop_stressing(args->name, SIGALRM)) {
+	if (stress_signal_stop_stressing(args->name, SIGALRM)) {
 		rc = EXIT_FAILURE;
 		goto die;
 	}
@@ -273,9 +280,9 @@ static int stress_socket_server(
 		goto die_close;
 	}
 
-	if (stress_set_sockaddr(args->name, args->instance, ppid,
-			AF_UNIX, socket_fd_port,
-			&addr, &addr_len, NET_ADDR_ANY) < 0) {
+	if (stress_net_sockaddr_set(args->name, args->instance, ppid,
+				AF_UNIX, socket_fd_port,
+				&addr, &addr_len, NET_ADDR_ANY) < 0) {
 		rc = EXIT_FAILURE;
 		goto die_close;
 	}
@@ -301,61 +308,82 @@ static int stress_socket_server(
 	do {
 		int sfd;
 
-		if (!keep_stressing(args))
+		if (UNLIKELY(!stress_continue(args)))
 			break;
 
 		sfd = accept(fd, (struct sockaddr *)NULL, NULL);
-		if (sfd >= 0) {
+		if (LIKELY(sfd >= 0)) {
 			ssize_t i;
 
-			for (i = 0; keep_stressing(args) && (i < max_fd); i++) {
+			for (i = 0; LIKELY(stress_continue(args) && (i < max_fd)); i++) {
 				int new_fd;
 
+#if defined(HAVE_SELECT)
+				if (socket_fd_reuse) {
+					fd_set readfds;
+					struct timeval tv;
+					int sret;
+
+					FD_ZERO(&readfds);
+					FD_SET(sfd, &readfds);
+					tv.tv_sec = 0;
+					tv.tv_usec = 0;
+
+					sret = select(sfd + 1, &readfds, NULL, NULL, &tv);
+					if (sret > 0) {
+						new_fd = stress_socket_fd_recv(sfd);
+					} else {
+						new_fd = open("/dev/zero", O_RDWR);
+					}
+				} else {
+					new_fd = open("/dev/zero", O_RDWR);
+				}
+#else
+				(void)socket_fd_reuse;
 				new_fd = open("/dev/zero", O_RDWR);
-				if (new_fd >= 0) {
+#endif
+
+				if (LIKELY(new_fd >= 0)) {
 					ssize_t ret;
 
-					ret = stress_socket_fd_sendmsg(sfd, new_fd);
+					ret = stress_socket_fd_send(sfd, new_fd);
 					if ((ret < 0) &&
-					     ((errno != EAGAIN) &&
-					      (errno != EINTR) &&
-					      (errno != EWOULDBLOCK) &&
-					      (errno != ECONNRESET) &&
-					      (errno != ENOMEM) &&
+					    ((errno != EAGAIN) &&
+					     (errno != EINTR) &&
+					     (errno != EWOULDBLOCK) &&
+					     (errno != ECONNRESET) &&
+					     (errno != ENOMEM) &&
 #if defined(ETOOMANYREFS)
-					      (errno != ETOOMANYREFS) &&
+					     (errno != ETOOMANYREFS) &&
 #endif
-					      (errno != EPIPE))) {
+					     (errno != EPIPE))) {
 						pr_fail("%s: sendmsg failed, errno=%d (%s)\n",
 							args->name, errno, strerror(errno));
 						(void)close(new_fd);
 						break;
 					}
 					(void)close(new_fd);
-					VOID_RET(ssize_t, stress_socket_fd_sendmsg(sfd, bad_fd));
+					VOID_RET(ssize_t, stress_socket_fd_send(sfd, bad_fd));
 					msgs++;
-					inc_counter(args);
+
+					stress_bogo_inc(args);
 				}
 			}
 			(void)close(sfd);
 		}
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 die_close:
 	(void)close(fd);
 die:
-#if defined(HAVE_SOCKADDR_UN)
+#if defined(HAVE_SYS_UN_H) &&	\
+    defined(HAVE_SOCKADDR_UN)
 	if (addr) {
-		struct sockaddr_un *addr_un = (struct sockaddr_un *)addr;
+		const struct sockaddr_un *addr_un = (struct sockaddr_un *)addr;
 
 		(void)shim_unlink(addr_un->sun_path);
 	}
 #endif
-
-	if (pid) {
-		(void)kill(pid, SIGALRM);
-		(void)shim_waitpid(pid, &status, 0);
-	}
 	pr_dbg("%s: %" PRIu64 " messages sent\n", args->name, msgs);
 
 	return rc;
@@ -365,19 +393,30 @@ die:
  *  stress_sockfd
  *	stress socket fd passing
  */
-static int stress_sockfd(const stress_args_t *args)
+static int stress_sockfd(stress_args_t *args)
 {
 	pid_t pid, mypid = getpid();
-	ssize_t max_fd = (ssize_t)stress_get_file_limit();
+	ssize_t max_fd = (ssize_t)stress_fs_file_limit_get();
 	int socket_fd_port = DEFAULT_SOCKET_FD_PORT;
 	int ret = EXIT_SUCCESS, reserved_port;
 	int *fds;
+	bool socket_fd_reuse = false;
 	size_t fds_size;
 
-	(void)stress_get_setting("sockfd-port", &socket_fd_port);
+	if (stress_signal_sigchld_handler(args) < 0)
+		return EXIT_NO_RESOURCE;
+
+	(void)stress_setting_get("sockfd-port", &socket_fd_port);
+	(void)stress_setting_get("sockfd-reuse", &socket_fd_reuse);
+
+#if !defined(HAVE_SELECT)
+	if ((socket_fd_reuse) && (stress_instance_zero(args)))
+		pr_inf("%s: select() is not supported, sockfd-reuse option is disabled\n", args->name);
+#endif
 
 	socket_fd_port += args->instance;
-	reserved_port = stress_net_reserve_ports(socket_fd_port, socket_fd_port);
+	socket_fd_port = stress_net_port_wraparound(socket_fd_port);
+	reserved_port = stress_net_reserve_ports(args, socket_fd_port, socket_fd_port);
 	if (reserved_port < 0) {
 		pr_inf_skip("%s: cannot reserve port %d, skipping stressor\n",
 			args->name, socket_fd_port);
@@ -395,26 +434,30 @@ static int stress_sockfd(const stress_args_t *args)
 	 */
 	if (geteuid() == 0) {
 		max_fd -= 64;
-		max_fd /= args->num_instances ? args->num_instances : 1;
+		max_fd /= args->instances ? args->instances : 1;
 		if (max_fd < 0)
 			max_fd = 1;
 	}
+	if (max_fd > (1024 * 1024))
+		max_fd = 1024 * 1024;
 
 	fds_size = sizeof(*fds) * (size_t)max_fd;
-	fds = malloc(fds_size);
+	fds = (int *)malloc(fds_size);
 	if (!fds) {
-		pr_inf_skip("%s: cannot allocate %zd file descriptors, skipping stressor\n",
-			args->name, max_fd);
+		pr_inf_skip("%s: failed to allocate %zd file descriptors%s, skipping stressor\n",
+			args->name, max_fd, stress_memory_free_get());
 		return EXIT_NO_RESOURCE;
 	}
 
-	stress_set_proc_state(args->name, STRESS_STATE_RUN);
+	stress_proc_state_set(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
+	stress_proc_state_set(args->name, STRESS_STATE_RUN);
 again:
 	pid = fork();
 	if (pid < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
-		if (!keep_stressing(args)) {
+		if (UNLIKELY(!stress_continue(args))) {
 			ret = EXIT_SUCCESS;
 			goto finish;
 		}
@@ -423,37 +466,40 @@ again:
 		free(fds);
 		return EXIT_FAILURE;
 	} else if (pid == 0) {
-		stress_set_oom_adjustment(args->name, false);
-		ret = stress_socket_client(args, mypid, max_fd, socket_fd_port, fds, fds_size);
-
-		/* Inform parent we're all done */
-		(void)kill(getppid(), SIGALRM);
-
+		stress_proc_state_set(args->name, STRESS_STATE_RUN);
+		stress_set_oom_adjustment(args, false);
+		stress_make_it_fail_set();
+		ret = stress_socket_client(args, mypid, max_fd, socket_fd_port, socket_fd_reuse, fds, fds_size);
 		_exit(ret);
 	} else {
-		ret = stress_socket_server(args, pid, mypid, max_fd, socket_fd_port);
+		int status;
+
+		ret = stress_socket_server(args, mypid, max_fd, socket_fd_port, socket_fd_reuse);
+		(void)shim_kill(pid, SIGALRM);
+		(void)shim_waitpid(pid, &status, 0);
 	}
 
 finish:
-	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
+	stress_proc_state_set(args->name, STRESS_STATE_DEINIT);
 
 	free(fds);
 
 	return ret;
 }
 
-stressor_info_t stress_sockfd_info = {
+const stressor_info_t stress_sockfd_info = {
 	.stressor = stress_sockfd,
-	.class = CLASS_NETWORK | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.classifier = CLASS_NETWORK | CLASS_OS,
+	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
-stressor_info_t stress_sockfd_info = {
+const stressor_info_t stress_sockfd_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_NETWORK | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.classifier = CLASS_NETWORK | CLASS_OS,
+	.opts = opts,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "only supported on Linux"
 };
